@@ -10,6 +10,7 @@ from __future__ import division
 
 import numpy as np
 from matplotlib import pyplot as plt
+from matplotlib import ticker
 from scipy import interpolate
 
 from wyrm import tentensystem as tts
@@ -25,12 +26,10 @@ def plot_scalp(v, channel):
     y = [i[1] for i in points]
     z = v
     X, Y, Z = interpolate_2d(x, y, z)
-    plt.clf()
-    vmin, vmax = 0, .5
-    plt.contour(X, Y, Z, 20, vmin=vmin, vmax=vmax)
-    plt.contourf(X, Y, Z, 20, vmin=vmin, vmax=vmax)
+    plt.contour(X, Y, Z, 20)
+    plt.contourf(X, Y, Z, 20)
     #plt.clabel(im)
-    #plt.colorbar()
+    plt.colorbar()
     plt.gca().add_artist(plt.Circle((0, 0), radius=1, linewidth=3, fill=False))
     plt.plot(x, y, 'bo')
     for i in zip(channel, zip(x,y)):
@@ -61,18 +60,30 @@ def plot_channels(dat, chanaxis=-1, otheraxis=-2):
 
 
 def plot_spatio_temporal_r2_values(dat):
+    """Calculate the signed r^2 values and plot them in a heatmap.
+
+    Paramters
+    ---------
+    dat : Data
+        epoched data
+
+    """
     r2 = proc.calculate_signed_r_square(dat)
     max = np.max(np.abs(r2))
     plt.imshow(r2.T, aspect='auto', interpolation='None', vmin=-max, vmax=max, cmap='RdBu')
-    plt.imshow(r2.T, aspect='auto', vmin=-max, vmax=max, cmap='RdBu')
+    ax = plt.gca()
     # TODO: sort front-back, left-right
-    chan_names = dat.axes[-1]
-    chan_idxs = range(len(chan_names))
-    plt.yticks(chan_idxs[::5], chan_names[::5])
-    xticks = ['%.1f' % i for i in dat.axes[-2]]
-    xtick_idxs = range(len(dat.axes[-2]))
-    plt.xticks(xtick_idxs[::5], xticks[::5])
+    # use the locators to fine-tune the ticks
+    #ax.yaxis.set_major_locator(ticker.MaxNLocator())
+    #ax.xaxis.set_major_locator(ticker.MaxNLocator())
+    ax.yaxis.set_major_formatter(ticker.IndexFormatter(dat.axes[-1]))
+    ax.xaxis.set_major_formatter(ticker.IndexFormatter(['%.1f' % i for i in dat.axes[-2]]))
+    plt.xlabel('%s [%s]' % (dat.names[-2], dat.units[-2]))
+    plt.ylabel('%s [%s]' % (dat.names[-1], dat.units[-1]))
+    plt.tight_layout(True)
     plt.colorbar()
+    plt.grid(True)
+
 
 def plot_spectrum(spectrum, freqs):
     plt.plot(freqs, spectrum, '.')
