@@ -15,11 +15,69 @@ import re
 import numpy as np
 import scipy as sp
 from scipy import signal
+from sklearn.covariance import LedoitWolf as LW
 
 logging.basicConfig(level=logging.NOTSET)
 logger = logging.getLogger(__name__)
 
 
+def lda_train(x, y, shrink=True):
+    """Train the LDA
+
+    Parameters
+    ----------
+    x : 2d array
+    y : 1d array
+    shrink : Boolean, optional
+
+    Returns
+    -------
+    w : 1d array
+    b : float
+
+    See Also
+    --------
+    lda_apply
+
+    """
+    assert len(np.unique(y)) == 2
+    mu1 = np.mean(x[y == 0], axis=0)
+    mu2 = np.mean(x[y == 1], axis=0)
+    # x' = x - m
+    m = np.empty(x.shape)
+    m[y == 0] = mu1
+    m[y == 1] = mu2
+    x2 = x - m
+    # w = cov(x)^-1(mu2 - mu1)
+    if shrink:
+        covm = LW().fit(x2).covariance_
+    else:
+        covm = np.cov(x2.T)
+    w = np.dot(np.linalg.pinv(covm), (mu2 - mu1))
+    # b = 1/2 x'(mu1 + mu2)
+    b = -0.5 * np.dot(w.T, (mu1 + mu2))
+    return w, b
+
+
+def lda_apply(clf, x):
+    """Apply LDA
+
+    Parameters
+    ----------
+    clf : (1d array, float)
+    x :
+
+    Returns
+    -------
+
+    See Also
+    --------
+    lda_train
+
+    """
+    w, b = clf
+    return np.dot(x, w) + b
+
 
 def swapaxes(dat, ax1, ax2):
     """Swap axes of a Data object.
@@ -1081,7 +1139,7 @@ def spectrum(dat, timeaxis=-2):
     return spm
 
 
-def spectrogram(cnt, timeaxis=-2):
+def spectrogram(cnt):
     """Calculate the spectrogram of a continuous data object.
 
     See Also
@@ -1089,15 +1147,14 @@ def spectrogram(cnt, timeaxis=-2):
     spectrum, stft
 
     """
-    framesize = 2000 #ms
-    width = int(cnt.fs * framesize / 1000)
-    specgram = stft(cnt.data, width, timeaxis=timeaxis)
-    freqs = sp.fftpack.fftfreq(width, 1/cnt.fs)[1:width/2]
-    print freqs
+    framesize = 1000 #ms
+    width = int(cnt.fs * (framesize / 1000))
+    specgram = np.array([stft(cnt.data[:,i], width) for i in range(cnt.data.shape[-1])])
+    freqs = sp.fftpack.rfftfreq(width, 1/cnt.fs)
     return specgram, freqs
 
 
-def stft(x, width, timeaxis=-2):
+def stft(x, width):
     """Short time fourier transform of a real sequence.
 
     This method performs a discrete short time Fourier transform. It
@@ -1127,10 +1184,9 @@ def stft(x, width, timeaxis=-2):
 
     """
     window = sp.hanning(width)
-    fourier = np.array([sp.fftpack.fft(x[i:i+width] * window)[1:width/2] for i in range(0, len(x)-width)])#, width//2)])
-    amps = 2 * fourier / width
-    amps = np.abs(amps)
-    return amps
+    fourier = np.array([sp.fftpack.rfft(x[i:i+width] * window) for i in range(0, len(x)-width, width//2)])
+    fourier *= (2 / width)
+    return fourier
 
 
 def calculate_csp(class1, class2):
